/*
 * kbd.c
 *
 *  Created on: 03.12.2013
 *      Author: alone
 */

#include <avr/io.h>
//#include <avr/iom8.h>
#include <avr/interrupt.h>//библиотека с прерываниями
#include <stdio.h>
#include <string.h>
#include <avr/sfr_defs.h>
#include <util/delay.h>

#include "kbd.h"
#include "hd44780.h"
#include "menu.h"
#include "transomot.h"
#include "save_params.h"
#include "stepper.h"
#include "rotator.h"

unsigned char key_tab[4] = { 0b11111110, 0b11111101, 0b11111011, 0b11110111 };
unsigned char key_codes[17] = { ' ', '3', '2', '1', '6', '5', '4', '9', '8',
    '7', 'B', '0', 'A', 'C', 'D', 'E', 'F' }; //матрица соответствия кодов клавиш

void kbd_setup(void) {
  TIMSK = (1 << TOIE0); //Разрешить прерывания по переполнению таймера0

  //TCCR0 = (1 << CS00) | (0 << CS01) | (0 << CS02); //Делитель =/1
  TCCR0 = (0 << CS00) | (1 << CS01) | (0 << CS02); //Делитель =/8
//  TCCR0 = (1 << CS00) | (1 << CS01) | (0 << CS02); //Делитель =/64
  //  TCCR0 = (0 << CS00) | (0 << CS01) | (1 << CS02); //Делитель =/256

  CONF_KBD_DDR |= (1 << CONF_KBD_P3) | (1 << CONF_KBD_P2) | (1 << CONF_KBD_P1) | (1 << CONF_KBD_P0); // Порт вывода

  CONF_KBD_DDR &= ~(1 << CONF_KBD_P7) | (1 << CONF_KBD_P6) | (1 << CONF_KBD_P5) | (1 << CONF_KBD_P4); // Порт ввода

  CONF_KBD_PORT = 0xF0; // Устанавливаем лог. 1 в порт ввода
}

// Функция опроса клавиатуры 4x4
unsigned char scan_key(void) {

  unsigned char key_value = 0;
  unsigned char i;

  for (i = 0; i < 4; i++) {
    CONF_KBD_PORT = key_tab[i]; // выводим лог. 0 в порт вывода
    _delay_us(5);

    switch (CONF_KBD_PIN & 0xF0) {

    case 0b11100000:
      key_value = 1 + i * 3;
      break;

    case 0b11010000:
      key_value = 2 + i * 3;
      break;

    case 0b10110000:
      key_value = 3 + i * 3;
      break;

    case 0b01110000:
      key_value = 4 + i * 3;
      break;

    default:
      break;
    }
  }
  return (key_value);
}

void scan_reductor_btn(void) {
  if (bit_is_clear(CONF_REDUCTOR_PIN, CONF_REDUCTOR_BTN)) {
    reductor_power(!reductor_power_state);
  }
}

/*
 *
 * <state> = 0|1
 * */
void reductor_power(uint8_t state) {
  switch (state) {
  case CONF_REDUCTOR_PWR_OFF:
//    poweroff
    CONF_REDUCTOR_PORT &= ~_BV(CONF_REDUCTOR_PWR);
    lcd_goto(1, 6);
    lcd_putc(' ');
    break;

  case CONF_REDUCTOR_PWR_ON:
//    poweron
    CONF_REDUCTOR_PORT |= _BV(CONF_REDUCTOR_PWR);
    lcd_goto(1, 6);
    lcd_putc('*');
    break;
  }

  reductor_power_state = state;
  sw_delay_ms(250);
}

ISR(TIMER0_OVF_vect) {
  unsigned char key = scan_key();

  switch (menu_state) {
  case MENU_WORK:
    kbd_handler_menu_work(key);
    break;

  case MENU_EDIT_WIRE:
    kbd_handler_menu_edit_wire(key);
    break;

  case MENU_EDIT_LAYER_COUNT:
    kbd_handler_menu_layer_count(key);
    break;

  default:
    break;
  }

  // scan start/stop reductor motor port
  scan_reductor_btn();
  //------------------------------------

  TCNT0 = 0x00; //Очищаем счетчик
  TIFR = 0x00; //Очищаем флаг переполнения

  return;
}

void kbd_handler_menu_work(unsigned char key) {
  if (key == 0) {
    return;
  }

  switch (key) {

  // вход в сетап
  case KEY_A:
    menu_state = MENU_EDIT_WIRE;
    lcd_goto(menu_diameter_pos[1], menu_diameter_pos[0]);
    lcd_cmd(DISPLAY_CTRL_DispOn_BlkOff_CrsOn, 10);
    sw_delay_ms(250);
    break;

    // очистка счетчиков
  case KEY_B:
    clear_counters();
    sw_delay_ms(250);
    break;

    // сохранение параметров в EEPROM
  case KEY_0:
    lcd_goto(0, 6);
    lcd_putc('W');
    save_parameters();
    sw_delay_ms(500);
    lcd_goto(0, 6);
    lcd_putc(' ');
    break;

    // точная (пошаговая) перемотка влево
  case KEY_1:
    slow_right();
    motor_free();
    break;

    // быстрая перемотка влево
  case KEY_4:
    right();
    sw_delay_ms(2);
    motor_free();
    break;

    // точная (пошаговая) перемотка вправо
  case KEY_3:
    slow_left();
    motor_free();
    break;

    // быстрая перемотка вправо
  case KEY_6:
    left();
    sw_delay_ms(2);
    motor_free();
    break;

    // смена направления
    //на счетчик не влияет!!
  case KEY_2:
    switch_direction();
    sw_delay_ms(250);
    break;

    // on/off автореверс
  case KEY_5:
    switch_reverse();
    sw_delay_ms(250);
    break;

    // перемотка каретки в начало слоя
  case KEY_7:
    carret_to_begin_layer_pos();
    sw_delay_ms(250);
    break;

    // Чтение данных из EEPROM
  case KEY_8:
    lcd_goto(0, 6);
    lcd_putc('R');
    restore_parameters();
    sw_delay_ms(300);
    lcd_goto(0, 6);
    lcd_putc(' ');

    show_diameter();
    show_layer_count();
    show_layer_counter();
    show_cycles_counter();
    show_direction();
    show_reverse_state();
    break;

    // перемотка каретки в конец слоя
  case KEY_9:
    carret_to_end_layer_pos();
    sw_delay_ms(250);
    break;

  default:
    break;
  }
}

void kbd_handler_menu_edit_wire(unsigned char key) {
  if (key == 0) {
    return;
  }

  switch (key) {
  // переход к следующему параметру
  case KEY_A:
    end_edit_wire();
    sw_delay_ms(250);
    break;

  case KEY_B:
    del_char(menu_diameter_pos, diameter);
    sw_delay_ms(250);
    break;

  default:
    get_char(menu_diameter_pos, diameter, key);
    sw_delay_ms(250);
    break;
  }
}

void kbd_handler_menu_layer_count(unsigned char key) {
  if (key == 0) {
    return;
  }

  switch (key) {
  case KEY_A:
    end_edit_layer_count();
    sw_delay_ms(250);
    break;

  case KEY_B:
    del_char(menu_layer_count, layer_count);
    sw_delay_ms(250);
    break;

  default:
    get_char(menu_layer_count, layer_count, key);
    sw_delay_ms(250);
    break;
  }
}

void get_char(uint8_t *menu_item, char *elem, unsigned char key) {
  uint8_t current_pos = menu_item[3];

  if (current_pos < menu_item[2]) {
    elem[current_pos] = key_codes[key];
    lcd_goto(menu_item[1], menu_item[0] + menu_item[3]);
    lcd_putc(key_codes[key]);
    menu_item[3]++;
    if (current_pos == 0 && menu_item[4] == MENU_EDIT_WIRE) {
      menu_item[3]++;
      lcd_goto(menu_item[1], menu_item[0] + menu_item[3]);
    }
  }
}

void del_char(uint8_t *menu_item, char *elem) {
  uint8_t current_pos = menu_item[3];

  if (current_pos > 0) {
    menu_item[3]--;
    if (current_pos == 2 && menu_item[4] == MENU_EDIT_WIRE) {
      menu_item[3]--;
    }
    lcd_goto(menu_item[1], menu_item[0] + menu_item[3]);
    elem[current_pos] = '0';
    lcd_putc(elem[current_pos]);
    lcd_goto(menu_item[1], menu_item[0] + menu_item[3]);
  }
}

// функция, вызываемая по окончанию ввода цифр диаметра
// перевод курсора на следующий редактируемый элемент,
// вычисление шага каретки
// преобразование строки в число
void end_edit_wire() {
  menu_state = MENU_EDIT_LAYER_COUNT;
  menu_diameter_pos[MENU_CURRENT_POS] = 0;
  diameter_to_int();
  show_diameter();
  lcd_goto(menu_layer_count[MENU_Y], menu_layer_count[MENU_X]);
}

void diameter_to_int() {
  diameter_dec = 0;
  int k = 3;
  for (int i = 0; i < strlen(diameter); i++) {
    if (i != 1) {
      diameter_dec += (diameter[i] - '0') * dec_pow(k);
      k--;
    }
  }
  step = diameter_dec / 5;
//  step = (diameter_dec/2)/2;
}

// функция, вызываемая по окончании ввода цифр витков слоя
// вывод отредактированного параметра на экран,
// преобразование строки в число
void end_edit_layer_count() {
  lcd_cmd(DISPLAY_CTRL_DispOn_BlkOff_CrsOff, 10); // cursor off
  menu_state = MENU_WORK; // work mode
  menu_layer_count[MENU_CURRENT_POS] = 0; // reset parameter pos

  uint8_t k = 2;
  layer_count_dec = 0;
  for (int i = 0; i < strlen(layer_count); i++) {
    layer_count_dec += (layer_count[i] - '0') * dec_pow(k);
    k--;
  }

  show_layer_count();
}
