/*
 * transomot.c
 *
 *  Created on: 10.07.2014
 *      Author: alone
 */

#include <avr/io.h>
#include <avr/iom8.h>

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <inttypes.h>

#include <util/delay.h>
#include <avr/interrupt.h>

#include "config.h"
#include "hd44780.h"
#include "kbd.h"
#include "stepper.h"
#include "menu.h"
#include "save_params.h"
#include "rotator.h"
#include "transomot.h"

int main(void) {
  cli();
  setup();
  sei();

  intro();
  initial_menu();
  worker();
}

void setup(void) {
  strcpy(diameter, "0.000");
  diameter_dec = 0;

//   кол-во витков в слое
  strcpy(layer_count, "000");
  layer_count_dec = 0;

//   счетчик витков в слое
  strcpy(layer_counter, "  0");
  layer_counter_dec = 0;

//   счетчик витков
  strcpy(cycles_counter, "   0");
  cycles_counter_dec = 0;

//   направление намотки
  direction = RIGHT_DIR;

//   состояние автореверса on/off
  reverse_state = 0;

//   кол-во импульсов шагового двигателя для смещения на 1 диаметр
  step = 0;

//  Инициализация нулевого таймера
  kbd_setup();

//  инициализация драйвера шагового двигателя
  stepper_init();

//   инициализация портов датчика оборотов
  reed_sensors_setup();

//   откл. питание редуктора
  reductor_power_state = 0;

  CONF_REDUCTOR_DDR &= ~_BV(CONF_REDUCTOR_BTN); // порт кнопки на чтение
  CONF_REDUCTOR_PORT |= _BV(CONF_REDUCTOR_BTN); // ставим подтяжку к "1"

  CONF_REDUCTOR_DDR |= _BV(CONF_REDUCTOR_PWR); // порт управления питанием на запись
  CONF_REDUCTOR_PORT |= _BV(CONF_REDUCTOR_PWR);
  sw_delay_ms(100);
  CONF_REDUCTOR_PORT &= ~_BV(CONF_REDUCTOR_PWR);

//   инициализация дисплея
  CONF_DISP_PORT = 0x00;
  CONF_DISP_DDR = 0xff; // PORT to output, for LCD drive
  lcd_init(); // init the LCD screen
}

void reed_sensors_setup(void) {
  CONF_REED_DDR &= ~_BV(CONF_REED_P0);  // пины для ввода
  CONF_REED_DDR &= ~_BV(CONF_REED_P1);

  CONF_REED_PORT |= _BV(CONF_REED_P0); // притягиваем к "1"
  CONF_REED_PORT |= _BV(CONF_REED_P1); // притягиваем к "1"
}

// обработчик датчиков оборотов редуктора
void worker(void) {

  uint8_t state0 = 0;
  uint8_t state1 = 0;
  uint8_t sumstate = 0;
  uint8_t sumstate_prev = 0;
  uint8_t step = 0;
  uint8_t rotation_direction = LEFT_DIR;

  while (1) {
//          обработка датчиков оборотов и вызов смещения каретки в нужную сторону
    state0 = !(CONF_REED_PIN & (1 << CONF_REED_P0));
    state1 = !(CONF_REED_PIN & (1 << CONF_REED_P1));
    sumstate = (state0 << CONF_REED_P0) | (state1 << CONF_REED_P1);

    if (sumstate == sumstate_prev) {
      continue; // в смысле, пропускаем ход, так как состояние не изменилось
    }

    sumstate_prev = sumstate;

    switch (step) {
//     STEP 0
    case 0:
      switch (sumstate) {
      case 1:
        rotation_direction = RIGHT_DIR;
        step = 1;
        break;

      case 2:
        rotation_direction = LEFT_DIR;
        step = 1;
        break;
      }
      break;

//       STEP 1
    case 1:
      if (sumstate == 3) {
        step = 2;
      }
      break;

//       STEP 2
    case 2:
      if (rotation_direction == LEFT_DIR) {
        if (sumstate == 1) {
          step = 0;
          rotate(rotation_direction);
        }
      } else {
        if (sumstate == 2) {
          step = 0;
          rotate(rotation_direction);
        }
      }
      break;
    }
  }

}

// заставка
void intro(void) {
  lcd_clrscr();
  lcd_goto(0, 1);
  lcd_prints("-=Transomot=-");
  lcd_goto(1, 1);
  lcd_prints("(c)Alone 2014");
  sw_delay_ms(2000);
}

// точная задержка в мс. Сдернуто с тырнета, работает правильно. Трогать нинада.
void sw_delay_ms(unsigned short count) {
//   Чтение задержки переменной до нуля
  while (count-- > 0) {
//     1 мс задержка специальной функции
    _delay_ms(1);
  }
}

int dec_pow(int pow) {
  int res = 1;
  for (int i = 0; i < pow; i++) {
    res *= 10;
  }
  return res;
}
